Deadened = LibStub("AceAddon-3.0"):NewAddon("Deadened", "AceHook-3.0", "AceTimer-3.0", "AceConsole-3.0", "AceEvent-3.0", "LibSink-2.0")

local AceConfig = LibStub("AceConfig-3.0")
local AceConfigDialog = LibStub("AceConfigDialog-3.0")
local bit_band = _G.bit.band

local announceModes = {None = "None", Group = "Group", Party = "Party", Raid = "Raid", Say = "Say", Yell = "Yell"}

local options = {
	type = "group",
	args = {
		spells = {
			type = "group",
			name = "Spells to suppress",
			desc = "Select the casts you want to suppress",
			args = {
				add = {
					type = "input",
					name = "Add a spell ID",
					desc = "Add the spell ID of a spell to suppress (must be numeric)",
					get = function() return "" end,
					set = function(info, v)
						Deadened.db.profile.spells[v] = true
						Deadened:AddSpell(v)
						Deadened:RebuildSpellTable()
					end,
					pattern = "%d+"
				}
			}
		},
		announce = {
			type = "select",
			name = "Announce",
			desc = "Announce interrupts to...",
			get = function()
				return Deadened.db.profile.announce
			end,
			set = function(info, v)
				Deadened.db.profile.announce = v
				Deadened:Print("Now announcing interrupts to", v)
			end,
			values = announceModes
		},
		modules = {
			type = "group",
			name = "Modules",
			desc = "Modules",
			args = {}
		}
	}
}
	
AceConfig:RegisterOptionsTable("Deadened", options, {"deaden", "deadened"})
Deadened:SetDefaultModuleState(false)

function Deadened:OnInitialize()
	self.db = LibStub("AceDB-3.0"):New("DeadenedDB")
	self.db:RegisterDefaults({
		profile = {
			modules = {},
			tankmode = false,
			sink20OutputSink = "None",
			announce = "None",
			spells = {
				[41426] = false,
				[41410] = true,
				[40832] = false,
				[40904] = false
			},
		}
	})
	optFrame = AceConfigDialog:AddToBlizOptions("Deadened", "Deadened")
	options.args.profile = LibStub("AceDBOptions-3.0"):GetOptionsTable(self.db)
	
	for k, v in self:IterateModules() do
		options.args.modules.args[k] = {
			type = "toggle", 
			name = k, 
			desc = "Toggle " .. k, 
			get = function()
				return Deadened.db.profile.modules[k] ~= false or false
			end,
			set = function(info, v)
				Deadened.db.profile.modules[k] = v
				if v then
					Deadened:EnableModule(k)
					Deadened:Print("Enabled", k, "module")
				else
					Deadened:DisableModule(k)
					Deadened:Print("Disabled", k, "module")
				end
			end
		}
	end		
end

local spellNames = {}

function Deadened:OnEnable()
	self:RegisterEvent("COMBAT_LOG_EVENT_UNFILTERED")
	for k, v in pairs(self.db.profile.spells) do
		self:AddSpell(k)
	end
	
	for k, v in self:IterateModules() do
		if self.db.profile.modules[k] ~= false then
			v:Enable()
		end
	end
end

function Deadened:AddSpell(k)
	local spellName = GetSpellInfo(k)
	spellNames[spellName] = v
	options.args.spells.args[spellName:gsub(" ", "")] = {
		type = "toggle",
		name = spellName,
		desc = spellName,
		get = function(info) return Deadened.db.profile.spells[k] end,
		set = function(info, val)
			Deadened.db.profile.spells[k] = val
			Deadened:RebuildSpellTable()
		end,
	}
end

function Deadened:RebuildSpellTable()
	for k, v in ipairs(self.db.profile.spells) do
		local spellName = GetSpellInfo(k)
		spellNames[spellName] = v
	end
end

function Deadened:InitModule(k)
	if self.db.profile.modules[k] ~= false then
		self:DisableModule(k)
		self:EnableModule(k)
	end
end

function Deadened:Dump()
	for k, v in pairs(dpsNames) do
		self:Print(k)
	end
	for k, v in pairs(tankNames) do
		self:Print(k)
	end
end

local function getDistribution(p)
	if p:lower() == "party" then
		if GetNumPartyMembers() > 0 then return "PARTY" end
	elseif p:lower() == "raid" then
		if GetNumRaidMembers() > 0 then return "RAID" end		
	elseif p:lower() == "group" then
		if GetNumRaidMembers() > 0 then return "RAID" end
		if GetNumPartyMembers() > 0 then return "PARTY" end
	elseif p:lower() == "say" then
		return "SAY"
	elseif p:lower() == "yell" then
		return "YELL"
	end
	return nil
end

local COMBATLOG_FILTER_ME = _G.COMBATLOG_FILTER_ME

function Deadened:COMBAT_LOG_EVENT_UNFILTERED(event, ...)
	local timestamp, eventtype, srcGUID, srcName, srcFlags, dstGUID, dstName, dstFlags = ...
	if eventtype == "SPELL_INTERRUPT" and srcGUID == UnitGUID("player") then
		local srcSpellId, srcSpellName, srcSpellSchool, dstSpellId, dstSpellName, dstSpellSchool = select(9, ...);

		if self.db.profile.announce == "None" then return end
		local d = getDistribution(self.db.profile.announce)
		if d then
			local msg = ("%sed %s's %s!"):format(srcSpellName, dstName, dstSpellName)
			SendChatMessage(msg, d)
		end	
	end
end

function Deadened:OnDisable()
end

function Deadened:IsDeaden(v)
	if type(v) == "number" then
		return self.db.profile.spells[v]
	end
	return spellNames[v]
end

local timer = nil
function Deadened:Alert(s)
	if timer then
		self:CancelTimer(timer, true)
		timer = nil
	end
	timer = self:ScheduleTimer(self.Pour, 0.25, self, s)
end

function Deadened:UNIT_SPELLCAST_START(orig_object, unit)
	if unit == "target" then
		local spell, rank, displayName, icon, startTime, endTime = UnitCastingInfo(unit)
		if not Deadened:IsDeaden(displayName) then
			for k,v in pairs(self.hooks[orig_object]) do
				v(orig_object, unit)
			end
		else
			local spell, rank, displayName, icon, startTime, endTime = UnitCastingInfo(unit)
			self:Alert(("%s casting %s (%s) sec"):format(UnitName(unit), displayName, ("%2.1f"):format((endTime-startTime) / 1000 )))
		end
	end	
end
